/**
 * @file  Channel4ModuleApplication.cpp
 * @brief Implementierung der Klasse CChannel4ModuleApplication.
 */
//////////////////////////////////////////////////////////////////////

//#include "StdAfx.h"
#include "Channel4ModuleApplication.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
void CChannel4ModuleApplication::InitClass(short channelID, short deviceID)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);

	// Anzahl anstehender Datensaetze
	srand(time(NULL));
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
CChannel4ModuleApplication::~CChannel4ModuleApplication() 
{
	while(m_records.size()) {

		m_records.pop();
	}
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleApplication::GetErrorModule(short module, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::GetErrorModule [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, error, text);

	// Ist IV_MODULE_APPLICATION ?
	if (module == IV_MODULE_APPLICATION) {

		switch(error) {
		case 1: sprintf(text, "( %i ): Das geforderte Modul wird nicht untersttzt.", error); break;
		case 2: sprintf(text, "( %i ): Die geforderte Funktion wird nicht untersttzt.", error); break;
		case 3: sprintf(text, "( %i ): Es wurden bei der Funktion zu wenige Paramteter angegeben.", error); break;
		case 4: sprintf(text, "( %i ): Es wurden bei der Funktion zu viele Paramteter angegeben.", error); break;
		default: sprintf(text, "( %i ): Nicht eingetragener Modulfehler!", error);
		}

		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleSystem::GetErrorModule(module, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleApplication::GetErrorFunction(short module, short function, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::GetErrorFunction [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, function, error, text);

	if (module == IV_MODULE_APPLICATION) {

		switch(function) {
		case IV_FUNCTION_CLEAR_RECORDS:
		case IV_FUNCTION_GET_RECORD:
		case IV_FUNCTION_SET_RECORD:
			switch(error) {
			case 1:sprintf(text, "( %i ): ", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SHOW_MESSAGE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Nachrichtentext zu lang fuer die Anzeige.", error); break;
			case 2:sprintf(text, "( %i ): Es wird bereits eine Nachricht angezeigt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SET_VARIABLE:
		case IV_FUNCTION_GET_VARIABLE:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Variablennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der zu setzende Variablenwert ist zu lang.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_SHOW_INFO:
			switch(error) {
			case 1:sprintf(text, "( %i ): Nachrichtentext zu lang fuer die Anzeige.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}
		
		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleSystem::GetErrorFunction(module, function, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::CallFunction(short module, short function, unsigned char *data, int &count)
{
	if (module == IV_MODULE_APPLICATION) {
		switch(function) {
		case IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK: return IV_ERROR_SUCCESS;
		case IV_FUNCTION_NOTIFY: return CallNotify(data, count);

		case IV_FUNCTION_CLEAR_RECORDS: return CallClearRecords(data, count);
		case IV_FUNCTION_SET_RECORD: return CallSetRecord(data, count);
		case IV_FUNCTION_GET_RECORD: return CallGetRecord(data, count);

		case IV_FUNCTION_SHOW_MESSAGE: return CallShowMessage(data, count);

		// Funktion nicht definiert
		default: return 0x0002; 
		}
	}

	return CChannel4ModuleSystem::CallFunction(module, function, data, count);
}

//#########################################################
//##################### APPLICATION #######################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::ClearRecords(short deviceIDRemote)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::ClearRecords [ %i ]\n", __FILE__, __LINE__, deviceIDRemote);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::ClearRecords [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::CallClearRecords(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::CallClearRecords [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	while (m_records.size()) {
		m_records.pop();
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::SetRecord(short deviceIDRemote, unsigned char &status)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::SetRecord [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, status);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD);
		return m_result;
	}

	// OK
	if (count > sizeof(unsigned char)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::SetRecord [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}
	
	switch(m_data[0]) {
	case 0:
	case 2:
		status = 0;
		break;
	case MB_ANSW_ACK_ON_RECORD:        // Quittung erhalten (fuer Online-Datensatz) 
	case MB_ANSW_TIMEOUT_ACK_RECORD:   // Quittung kam nach Timeout (fuer Online-Datensatz) 
		status = 1;
		break;
	case MB_ANSW_OLD_ACK_ON_RECORD:    // wiederholt gesendete Quittung 
	case MB_ANSW_OLD_TIMEOUT_ACK_RECORD: // wiederholte Quittung nach Timeout 
		status = 2;
		break;
	case MB_ANSW_ACK_OFF_RECORD:       // Quittung erhalten (fuer Offline-Datensatz) 
		status = 3;
		break;
	case MB_ANSW_OLD_ACK_OFF_RECORD:   // wiederholt gesendete Quittung 
		status = 4;
		break;
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::CallSetRecord(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::CallSetRecord [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	if (m_records.size()) {

		CRecord &record = m_records.front();

		switch(record.m_data[0]) {
		case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
			data[0] = 0;
			break;
		case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
		case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
			data[0] = MB_ANSW_ACK_ON_RECORD;
			break;
		case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
		case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
			data[0] = MB_ANSW_ACK_OFF_RECORD;
			break;
		}
		
		count = sizeof(unsigned char);

		// Datensatz loeschen
		m_records.pop();
	} else {

		data[0] = 0;
		count = sizeof(unsigned char);
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::GetRecord(short deviceIDRemote, unsigned char *buf, int &count, unsigned char &status)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::GetRecord [ %i, 0x08X, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, buf, count, status);

	int i;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD);
		return m_result;
	}

	// OK
	
	switch(m_data[0]) {
	case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
		status = 0;
		break;
	case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
		status = 1;
		break;
	case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
		status = 2;
		break;
	case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
		status = 3;
		break;
	case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
		status = 4;
		break;
	}
	
	if (((status == 0) && (count > 1)) || (count > 243)) {
		
		return -IV_ERROR_PROCESSING_DLL;
	}


	for (i=0; i < count; i++) {
		
		buf[i] = m_data[i];
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::CallGetRecord(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::CallGetRecord [ 0x08X, %i ]\n", __FILE__, __LINE__, data, count);

	count = 0;

	if (m_records.size()) {

		CRecord &record = m_records.front();

		// Daten kopieren
		data[0] = record.m_status;
		memcpy(&data[1], record.m_data, record.m_length);
		count = 1 + record.m_length;
	} else {

		data[count++] = MB_ANSW_NO_RECORD;
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::ShowMessage(short deviceIDRemote, unsigned char time, unsigned char action, unsigned char audio, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::ShowMessage [ %i, %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, time, action, audio, text);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	m_data[count] = time;
	count += sizeof(unsigned char);
	m_data[count] = action;
	count += sizeof(unsigned char);
	m_data[count] = audio;
	count += sizeof(unsigned char);

	// Namen uebernehmen
	count += sprintf((char*)&m_data[count], "%s", text) + 1;
	
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::ShowMessage [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::CallShowMessage(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::CallShowMessage [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	return CallFunctionNotify(IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE, data, count);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::SetVariable(short deviceIDRemote, char *name, char *value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::SetVariable [ %i, 0x08X, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, name, value);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Namen uebernehmen
	count = sprintf((char*)m_data, "%s", name) + 1;
	// Wert uebernehmen
	count += sprintf((char*)&m_data[count], "%s", value) + 1;
	
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::SetVariable [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::GetVariable(short deviceIDRemote, char *name, char *value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::GetVariable [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, name, value);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Namen uebernehmen
	count = sprintf((char*)m_data, "%s", name) + 1;
	
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count > (17 + 81)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::GetVariable [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Namen uebergehen
	count = strlen(name) + 1;

	// OK
	strcpy(value, (char*)&m_data[count]);

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::ShowInfo(short deviceIDRemote, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::ShowInfo [ %i, %i, %i, %i, %i, 0x%04X, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, maschine, time, font, audio, flags, text);

	int count = 0;
// Maschinennummer (1 Byte)
// Time ist die Anzeigedauer. (1 Byte) (0 im Hindergrund uebernehmen, -1 staendig anzeigen)
// Schriftgroessennummer (1 Byte)
// Akkustisches Signal (1 Byte)
// 16bit reserve (2 Byte)
// Flags z.b. 0x0001 (2 Byte) fuer Antiproportionale Schrift
// Infotext
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	m_data[count] = maschine;
	count += sizeof(unsigned char);
	m_data[count] = time;
	count += sizeof(unsigned char);
	m_data[count] = font;
	count += sizeof(unsigned char);
	m_data[count] = audio;
	count += sizeof(unsigned char);
	*(short*)&m_data[count] = 0;
	count += sizeof(short);
	*(short*)&m_data[count] = flags;
	count += sizeof(short);

	// Namen uebernehmen
	count += sprintf((char*)&m_data[count], "%s", text) + 1;
	
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::ShowInfo [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::Entrance2Identification(short deviceIDRemote, char *TM, char *Id, char *Pin, char *status, int &statusLength)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::Entrance2Identification [ %i, %s, %s, %s ]\n", __FILE__, __LINE__, deviceIDRemote, TM, Id, Pin);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Namen uebernehmen
	count = sprintf((char*)&m_data[count], TM) + 1;
	count += sprintf((char*)&m_data[count], Id) + 1;
	count += sprintf((char*)&m_data[count], Pin) + 1;
	
	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::Entrance2Identification [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	count = 0;

	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK);
		return m_result;
	}

	if (count == 0) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::Entrance2Identification [ Aktiver Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Wert von Status uebernehmen
	int i;
	for (i=0; (i < count) && (i < statusLength); i++) {

		status[i] = m_data[i];
	}
	// Uebergebene Laenge mitteilen
	statusLength = i - 1;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::Notify(short deviceIDRemote, unsigned char type, int priority, unsigned char deviceType, unsigned long serialNum, char *commString, int commValue, char *params)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::Notify [ %i, %i, %i, %s, %i, %s ]\n", __FILE__, __LINE__, deviceIDRemote, type, priority, commString, commValue, params);

	int count = 0;
	
	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	// Benachrichtigungstyp uebernehmen
	m_data[count] = type;
	count += sizeof(unsigned char);
	// Prioritaet eintragen
	*(short*)&m_data[count] = priority;
	count += sizeof(short);
	// Geraetetyp uebernehmen
	m_data[count] = deviceType;
	count += sizeof(unsigned char);
	// Seriennummer uebernehmen
	*(unsigned long*)&m_data[count] = serialNum;
	count += sizeof(unsigned long);
	// commString eintragen (IP oder Hostnamen)
	count += sprintf((char*)&m_data[count], "%s", commString) + 1;
	// commValue eintragen (Portnummer)
	*(unsigned short*)&m_data[count] = commValue;
	count += sizeof(short);
	// Zeitstempel
	time_t tm_time;
	struct tm *ptm;
	tm_time = time(NULL);
	ptm = localtime(&tm_time);
	count += sprintf((char*)&m_data[count], "%02i%02i-%02i-%02i %02i:%02i:%02i:%03i", 
		(ptm->tm_year + 1900) / 100,
		(ptm->tm_year + 1900) % 100,
		ptm->tm_mon + 1,
		ptm->tm_mday,
		ptm->tm_hour,
		ptm->tm_min,
		ptm->tm_sec, 0) + 1;
	// Leerstring
	count += sprintf((char*)&m_data[count], "%s", params) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY);
		return m_result;
	}

	// OK
	if (count > sizeof(int)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleApplication::Notifiy [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::NotifyRecord(short deviceIDRemote, int priority, unsigned char deviceType, unsigned long serialNum, char *commString, int commValue)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleApplication::NotifyRecord [ %i, %i, %i, %i, %s, %i ]\n", __FILE__, __LINE__, deviceIDRemote, priority, deviceType, serialNum, commString, commValue);

	return Notify(deviceIDRemote, IV_VALUE_NOTIFY_RECORD, priority, deviceType, serialNum, commString, commValue, "");
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleApplication::CallNotify(unsigned char *data, int &count)
{
	// Benachrichtigung an abgeleitete Klassen weiterleiten
	int res = CallFunctionNotify(IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY, data, count);

	// Daten loeschen
	count = 0;

	return res;
}
