/*!
    \file   df_lua_files.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Deklarationen der Funktionen für das Übertragen und das Bearbeiten von Dateien.
*/
#ifndef DF_LUA_FILES_H
#define DF_LUA_FILES_H

#include "df_lua_api.h"

#if (DF_API_VERSION >= 0x04030600)
/*! 
    \brief Schreiben einer Setupdatei.

    Die Setupdatei (*.aes; *.ael) wird auf das Gerät übertragen. Danach müssen ggf. die Listen neu übertragen werden. Ebenfalls ggf. die Zutrittslisten oder Timeboylisten.
    Es dürfen sich keine abrufbare Datensätze mehr auf dem Gerät befinden.

    \param[in] connectionId Verbindungs-Id. Diese erhalten Sie ggf. mittels DFCComOpenIV() oder per Active-Mode.
    \param[in] deviceAddress Addresse eines Gerätes im RS485 Bus, andernfalls 0.
    \param[in] filePath Die zu übertragende Setupdatei mit einer der Endungen *.aes oder *.ael.
    \param[out] errorNumber Variable für einen Fehlercode, falls die Ausführung scheitert.

    \return 1 wenn Erfolgreich, andernfalls 0. Weitere Informationen siehe \link errorNumber Fehlernummer. \endlink
*/
int l_writeFile( lua_State *L );

/*! 
    \brief Lesen einer Setupdatei.

    Die im Gerät enthaltene Setupdatei wird ausgelesen und in der angegebenen Datei gespreichert. 

    \param connectionId Verbindungs-Id. Diese erhalten Sie ggf. mittels DFCComOpenIV() oder per Active-Mode..
    \param deviceAddress Busnummer des anzusprechenden Geraetes.
    \param filePath Zeiger auf die Zeichenfolge, welche den Pfad und/oder Dateinamen enthaelt.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return 1 wenn Erfolgreich, andernfalls 0.

    \version Diese Funktion wurde in der Version 02.00.00 hinzugefügt.
*/
int l_readFile( lua_State *L );

/*! 
    \brief Lesen einer Setupdatei.

    Die im Gerät enthaltene Setupdatei wird ausgelesen und in der angegebenen Datei gespreichert. 

    \param connectionId Verbindungs-Id. Diese erhalten Sie ggf. mittels DFCComOpenIV() oder per Active-Mode..
    \param deviceAddress Busnummer des anzusprechenden Geraetes.
    \param filePath Zeiger auf die Zeichenfolge, welche den Pfad und/oder Dateinamen enthaelt.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return 1 wenn Erfolgreich, andernfalls 0.

    \version Diese Funktion wurde in der Version 02.00.00 hinzugefügt.
*/
int l_editFile( lua_State *L );

#endif

#endif
