#include "../../DatafoxLibraryIV.h"
#include "../df_connection.h"

DF_API_EXTERN HANDLE DF_API_CALL DFCGetComPort( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return INVALID_HANDLE_VALUE;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    HANDLE hRes = pChannel->getComPort();

    pChannel->logEnd("\t(%i)", hRes);

    return hRes;
}

DF_API_EXTERN int DF_API_CALL DFCSetComPort( int connectionId, HANDLE handle, int isSocket, int value, int timeout )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    int iRes = (pChannel->setComPort(handle, isSocket, value, timeout) == IV_ERROR_SUCCESS);

    pChannel->logEnd("\t%i\t%i\t%i\t%i\t(%i)", handle, isSocket, value, timeout, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComOpenSerial( int connectionId, char *commPort, int baudRate, int timeout )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%s\t%i\t%i", __func__, connectionId,
        (commPort == NULL) ? "" : commPort, baudRate, timeout);

    int iRes = FALSE;

    // Is Channel not engaged?
    if (!pChannel->isOpen())
    {
        if (pChannel->openAEIII(1, commPort, baudRate, timeout) == IV_ERROR_SUCCESS)
        {
            iRes = TRUE;
        }
    }
    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComOpenSocket( int connectionId, char *hostAddress, int portNumber, int timeout )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%s\t%i\t%i", __func__, connectionId,
        (hostAddress == NULL) ? "" : hostAddress, portNumber, timeout);

    // zulaessige Bereiche pruefen
    if (hostAddress == NULL )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iRes = FALSE;

    // Is Channel not engaged?
    if (!pChannel->isOpen())
    {
        if (pChannel->openAEIII(3, hostAddress, portNumber, timeout) == IV_ERROR_SUCCESS)
        {
            iRes = TRUE;
        }
    }

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComOpenIV( int connectionId, int deviceAddress, int connectionType, char *connectionString, int connectionValue, int connectionTimeout )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    if ( pChannel->tryEnterCriticalSection() == 0 )
    {
#ifdef _MSC_VER
        pChannel->logBegin( "%s.ERROR\t\"Object in use. Owner-Id: %u\"", __func__, pChannel->_threadIdOpen );
#else
        pChannel->logBegin( "%s.ERROR\t\"Object in use.\"" );
#endif
        pChannel->logEnd( "" );
        pChannel->enterCriticalSection();
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%s\t%i\t%i", __func__, connectionId, deviceAddress, connectionType,
        (connectionString == NULL) ? "" : connectionString, connectionValue, connectionTimeout);

    // zulaessige Bereiche pruefen
    if (((connectionType != 1) && (connectionType != 3) && (connectionType != 4)) 
        || (connectionString == NULL))
    {
        pChannel->logEnd("\t(0)");
        pChannel->leaveCriticalSection();
        return 0;
    }


    int iRes = FALSE;

    // Is Channel not engaged?
    if (!pChannel->isOpen())
    {
        //2009_09_02: Busnummer fix auf 32
        //2012_05_21: Busnummer fix auf 0 (wegen Mehrfachdocking TimeboyIV)
        if (pChannel->open(0, connectionType, connectionString, connectionValue, connectionTimeout) == IV_ERROR_SUCCESS)
        {
            iRes = TRUE;
#ifdef _MSC_VER
            // Die Thread-Id halten und beim Schlieen prfen,
            // dass auch der gleiche Thread wieder die Verbindung beendet.
            pChannel->_threadIdOpen = GetCurrentThreadId();
#endif
        }
    }

    pChannel->logEnd("\t(%i)", iRes);
    pChannel->leaveCriticalSection();

    return iRes;
}

DF_API_EXTERN void DF_API_CALL DFCComClose( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return;
    }

#ifdef _MSC_VER
    pChannel->logBegin("%s\t%i", __func__, connectionId);

    if ( pChannel->_threadIdOpen != 0 && pChannel->_threadIdOpen != GetCurrentThreadId() )
    {
        pChannel->logBegin( "%s.ERROR\t\"Not object owner. Owner-Id: %u\"", __func__, pChannel->_threadIdOpen );
        pChannel->logEnd( "" );
    }
    else
    {
        pChannel->close();
        pChannel->_threadIdOpen = 0;
    }

    pChannel->logEnd("");
#else
    pChannel->logBegin("%s\t%i", __func__, connectionId);

    pChannel->close();

    pChannel->logEnd("");
#endif
}

DF_API_EXTERN int DF_API_CALL DFCWrite( int connectionId, char *buffer, int toWrite, int *written, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if ((buffer == NULL) || (written == NULL) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t\t%i\t(0)", toWrite, IV_ERROR_FUNCTION_PARAMETER);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = FALSE;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t\t%i\t(0)", toWrite, *errorNumber);
        return FALSE;
    }

    if (pChannel->write(buffer, toWrite, written) == IV_ERROR_SUCCESS)
    {
        iRes = TRUE;
    }

    pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", (strlen(buffer) <= (unsigned int)toWrite) ? buffer : "",
        toWrite, *written, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCRead( int connectionId, char *buffer, int toRead, int *read, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if ((buffer == NULL) || (read == NULL) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t\t%i\t(0)", toRead, IV_ERROR_FUNCTION_PARAMETER);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t\t%i\t(0)", toRead, *errorNumber);
        return 0;
    }

    int iRes = FALSE;

    if (pChannel->read(buffer, toRead, read) == IV_ERROR_SUCCESS)
    {
        iRes = TRUE;
    }

    pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", (strlen(buffer) <= (unsigned int)toRead) ? buffer : "",
        toRead, *read, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCIsChannelOpen( int connectionId )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        return 0;
    }

    int iRes = -1;
    pChannel->logBegin("%s\t%i", __func__, connectionId);

    if (connectionId > 0 && connectionId < MAX_CHANNEL && (theApp.m_channelArray[connectionId] != NULL)) 
    {
        if (theApp.m_channelArray[connectionId]->isOpen()) 
        {
            iRes = 1;
        } 
        else 
        {
            iRes = 0;
        }
    }

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComUseHandle( int connectionId, HANDLE commHandle )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, commHandle);

    int iRes = (pChannel->comUseHandle(commHandle) == IV_ERROR_SUCCESS);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComReleaseHandle( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    int iRes = (pChannel->comReleaseHandle() == IV_ERROR_SUCCESS);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComInit( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    char szComStr[14];

#ifdef _MSC_VER
    if (connectionId > 9)
    {
        sprintf(szComStr, "\\\\.\\COM%i", connectionId);
    }
    else
    {
        sprintf(szComStr, "COM%i", connectionId);
    }

#else
    sprintf(szComStr, "/dev/ttyS%i", connectionId - 1);
#endif

    int iRes = FALSE;
    // Is Channel not engaged?
    if (!pChannel->isOpen())
    {
        if (pChannel->openAEIII(1, szComStr, 38400, 800) == IV_ERROR_SUCCESS) {

            iRes = TRUE;
        }
    }

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComOpen( int connectionId, char *hostAddress )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%s", __func__, connectionId, (hostAddress == NULL) ? "" : hostAddress);

    if (hostAddress == NULL )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iRes = FALSE;
    // Is Channel not engaged?
    if (!pChannel->isOpen())
    {
        if (pChannel->openAEIII(3, hostAddress, 8000, 5000) == IV_ERROR_SUCCESS)
        {
            iRes = TRUE;
        }
    }

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

