#include "../df_lua_setup_records.h"

//DF_API_EXTERN int DF_API_CALL DFCLoadDatensatzbeschreibung( int connectionId, int deviceAddress, int *errorNumber )
int l_readRecordTables( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );
    // Device
    int deviceAddress = static_cast<int>( luaL_checkinteger( L, 2 ) );
    luaL_argcheck( L, deviceAddress >= 1 && deviceAddress <= 254, 2, "out of range" );

    int res = 1;

    lua_pushboolean( L, DFCLoadDatensatzbeschreibung( lc->id, deviceAddress, &lc->errorNumber ) );

    if ( lua_toboolean( L, -1 ) == false )
        push_errorNumber( lc, res );

    return res;
}

//DF_API_EXTERN int DF_API_CALL DFCDatBCnt( int connectionId )
int l_getRecordTableCount( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );

    lua_pushinteger( L, DFCDatBCnt( lc->id ) );

    return 1;
}

//DF_API_EXTERN int DF_API_CALL DFCDatBDatensatz( int connectionId, int deviceAddress, unsigned char *name, int *fieldCount )
int l_getRecordTable( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );
    // Tabellennummer
    int tableNum = static_cast<int>( luaL_checkinteger( L, 2 ) );
    luaL_argcheck( L, tableNum >= 0 && tableNum <= 19, 2, "out of range" );

    int res = 1;
    std::string tableName( 64, '\0' );
    int fieldCount;

    lua_pushboolean( L, DFCDatBDatensatz( lc->id, tableNum, reinterpret_cast<unsigned char *>( const_cast<char *>( tableName.data() ) ), &fieldCount ) );
    if ( lua_toboolean( L, -1 ) == 1 )
    {
        lua_pushlstring( L, tableName.data(), tableName.find_first_of( '\0' ) );
        res++;

        lua_pushinteger( L, fieldCount );
        res++;
    }

    return res;
}

//DF_API_EXTERN int DF_API_CALL DFCDatBFeld( int connectionId, int dataBNumber, int number, unsigned char *name, int *type, int *size )
int l_getRecordField( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );
    // Tabellennummer
    int tableNum = static_cast<int>( luaL_checkinteger( L, 2 ) );
    luaL_argcheck( L, tableNum >= 0 && tableNum <= 19, 2, "out of range" );
    // Feldnummer
    int fieldNum = static_cast<int>( luaL_checkinteger( L, 3 ) );
    luaL_argcheck( L, fieldNum >= 0 && fieldNum <= 24, 2, "out of range" );

    int res = 1;
    std::string fieldName( 64, '\0' );
    int fieldType;
    int fieldSize;

    lua_pushboolean( L, DFCDatBFeld( lc->id, tableNum, fieldNum, reinterpret_cast<unsigned char *>( const_cast<char *>( fieldName.data() ) ), &fieldType, &fieldSize ) );
    if ( lua_toboolean( L, -1 ) == true )
    {
        lua_pushlstring( L, fieldName.data(), fieldName.find_first_of( '\0' ) );
        res++;

        lua_pushinteger( L, fieldType );
        res++;

        lua_pushinteger( L, fieldSize );
        res++;
    }

    return res;
}

//DF_API_EXTERN int DF_API_CALL DFCReadRecord( int connectionId, int deviceAddress, char *buffer, int *bufferLength, int *errorNumber )
int l_readRecord( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );
    // Device
    int deviceAddress = static_cast<int>( luaL_checkinteger( L, 2 ) );
    luaL_argcheck( L, deviceAddress >= 1 && deviceAddress <= 254, 2, "out of range" );

    int res = 1;
    std::string buffer( 255, '\0' );
    int bufferSize = buffer.size();

    lua_pushinteger( L, DFCReadRecord( lc->id, deviceAddress, const_cast<char *>( buffer.data() ), &bufferSize, &lc->errorNumber ) );

    if ( lua_tointeger( L, -1 ) == -1 )
    {
        lua_pushnil( L );
        res++;

        push_errorNumber( lc, res );
    }
    else
    {
        lua_pushlstring( L, buffer.data(), bufferSize );
        res++;
    }

    return res;
}

//DF_API_EXTERN int DF_API_CALL DFCQuitRecord( int connectionId, int deviceAddress, int *errorNumber )
int l_quitRecord( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );
    // Device
    int deviceAddress = static_cast<int>( luaL_checkinteger( L, 2 ) );
    luaL_argcheck( L, deviceAddress >= 1 && deviceAddress <= 254, 2, "out of range" );

    int res = 1;

    lua_pushinteger( L, DFCQuitRecord( lc->id, deviceAddress, &lc->errorNumber ) );

    if ( lua_tointeger( L, -1 ) == -1 )
        push_errorNumber( lc, res );

    return res;
}

//DF_API_EXTERN int DF_API_CALL DFCComClearData( int connectionId, int deviceAddress )
int l_resetRecords( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );
    // Device
    int deviceAddress = static_cast<int>( luaL_checkinteger( L, 2 ) );
    luaL_argcheck( L, deviceAddress >= 1 && deviceAddress <= 254, 2, "out of range" );

    int res = 1;

    lua_pushboolean( L, DFCComClearData( lc->id, deviceAddress ) );

    if ( lua_toboolean( L, -1 ) == false )
    {
        lc->errorNumber = IV_ERROR_UNREACHABLE;
        push_errorNumber( lc, res );
    }

    return res;
}

//DF_API_EXTERN int DF_API_CALL DFCRestoreRecords( int connectionId, int deviceAddress, int *errorNumber )
int l_restoreRecords( lua_State *L )
{
    // Channel-Data
    LuaChannel *lc = checkchannel( L );
    // Device
    int deviceAddress = static_cast<int>( luaL_checkinteger( L, 2 ) );
    luaL_argcheck( L, deviceAddress >= 1 && deviceAddress <= 254, 2, "out of range" );

    int res = 1;

    lua_pushboolean( L, DFCRestoreRecords( lc->id, deviceAddress, &lc->errorNumber ) );

    if ( lua_toboolean( L, -1 ) == false )
        push_errorNumber( lc, res );

    return res;
}
