/**
 * @file  AE3Buffer.cpp
 * @brief Implementierung der 32bit/16bit CRC - Routinen.
 */
//////////////////////////////////////////////////////////////////////

#include "crc.h"

unsigned int crc32_table[256]; // Lookup table array

// Reflection is a requirement for the official CRC-32 standard.
// You can create CRCs without it, but they won't conform to the standard.
unsigned int Reflect(unsigned int ref, char ch)
{// Used only by InitCRC32Table()

	unsigned int val = 0;

	// Swap bit 0 for bit 7
	// bit 1 for bit 6, etc.
	int i;
	for(i=1; i < (ch + 1); i++)
	{
		if(ref & 1)
			val |= 1 << (ch - i);
		ref >>= 1;
	}

	return val;
}

// Call this function only once to initialize the CRC table.
void InitCRC32Table() 
{// Called by Init

	// This is the official polynomial used by CRC-32 
	// in PKZip, WinZip and Ethernet. 
	unsigned int ulPolynomial = 0x04c11db7;

	// 256 values representing ASCII character codes.
	int i, j;
	for(i=0; i <= 0xFF; i++)
	{
		crc32_table[i]=Reflect(i, 8) << 24;
		for (j=0; j < 8; j++)
			crc32_table[i] = (crc32_table[i] << 1) ^ (crc32_table[i] & (1 << 31) ? ulPolynomial : 0);
		crc32_table[i] = Reflect(crc32_table[i], 32);
	}
}

// This function uses the crc32_table lookup table
// to generate a CRC for ucData
unsigned int GetCRC32(unsigned char *data, unsigned int size, unsigned int crc, unsigned char closure)
{
	// Be sure to use unsigned variables,
	// because negative values introduce high bits
	// where zero bits are required.
	//unsigned int crc = 0xffffffff;
	int len;
	unsigned char *buffer;

	len = size;
	// Save the text in the buffer.
	buffer = data;
	// Perform the algorithm on each character
	// in the string, using the lookup table values.
	while(len--)
		crc = (crc >> 8) ^ crc32_table[(crc & 0xFF) ^ *buffer++];
	// Exclusive OR the result with the beginning val.
	return (closure) ? crc^0xffffffff : crc;
}

// CRC 16bit berechnen
unsigned short GetCRC16(unsigned char *data, unsigned int size, unsigned short crc)
{
	unsigned int i;
	unsigned char j;
	//unsigned short crc;
  
	//crc = 0xFFFF;
    for (i=0; i < size; i++) {

        crc = crc ^ data[i];

        for (j=0; j < 8; j++) {
            if (crc & 0x1) {
				crc >>= 1; 
				crc = crc^0xA001;
			} else {
				crc >>= 1;
			}
        }
    }

    //data[size    ] = (unsigned char)(crc & 0xFF);
    //data[size + 1] = (unsigned char)(crc >> 8);
	return crc;
}

